#pragma once

#include "ScriptletContainerFactory.h"
#include "ComponentPropertyAccessor.h"
#include "ValueConverter.h"
#include "DataContext.h"


interface IPropertyBindingListener
{
public:
	virtual ~IPropertyBindingListener() {}
	virtual void OnPropertyChanged(LPCTSTR propertyKey, IDataContext* dataContext) = 0;
	virtual void OnPropertyChanged(LPCTSTR propertyKey, CComVariant& propertyValue) = 0;
};

class PropertyBinder
{
public:
	PropertyBinder() {}
	virtual ~PropertyBinder() {}

	void AddPropertyBindingListener(IPropertyBindingListener* listener)
	{
		ATLASSERT(listener != NULL);
		ATLASSERT(m_PropertyBindingListeners.Find(listener) == NULL);

		m_PropertyBindingListeners.AddTail(listener);
	}
	void RemovePropertyBindingListener(IPropertyBindingListener* listener)
	{
		ATLASSERT(listener != NULL);
		POSITION pos = m_PropertyBindingListeners.Find(listener);
		if (pos)
		{
			m_PropertyBindingListeners.RemoveAt(pos);		
		}
	}
	void FirePropertyBindingListener(LPCTSTR propertyKey, IDataContext* dataContext)
	{
		POSITION pos = m_PropertyBindingListeners.GetHeadPosition();
		while (pos)
		{
			IPropertyBindingListener* listener = m_PropertyBindingListeners.GetNext(pos);
			if (listener)
			{
				listener->OnPropertyChanged(propertyKey, dataContext);
			}
		}
	}
	void FirePropertyBindingListener(LPCTSTR propertyKey, CComVariant& propertyValue)
	{
		POSITION pos = m_PropertyBindingListeners.GetHeadPosition();
		while (pos)
		{
			IPropertyBindingListener* listener = m_PropertyBindingListeners.GetNext(pos);
			if (listener)
			{
				listener->OnPropertyChanged(propertyKey, propertyValue);
			}
		}
	}
protected:
	CAtlList<IPropertyBindingListener*> m_PropertyBindingListeners;
};

class Binding : public Object, public IPropertyBindingListener
{
public:
	Binding(void) 
	{
		m_Target = NULL;
		m_Source = NULL;
		m_ValueConverter = NULL;
	}
	virtual ~Binding(void)
	{
		Close();
	}

	//operation
	void SetKey(int key)
	{
	}
	void SetSource(IDataContext* dataContext, LPCTSTR sourcePropertyName)
	{
		ATLASSERT(dataContext);

		CString propertyKey = sourcePropertyName;
		IDataContext* childDataContext = dataContext->GetChildDataContext(propertyKey);
		m_DataContext = childDataContext;
		m_Source = m_DataContext->GetBinder();
		if (propertyKey.Find(_T('.')) >= 0)
		{
			m_SourceProperty.Format(_T("(%s)"), propertyKey);
		}
		else
		{
			m_SourceProperty = propertyKey;
		}
		

		m_Source->AddPropertyBindingListener(this);
	}
	void SetTarget(CComponent* target, LPCTSTR targetPropertyName)
	{
		ATLASSERT(target != NULL && _tcslen(targetPropertyName) > 0);
		m_Target = target;
		m_TargetProperty = targetPropertyName;		
	}
	void SetValueConverter(IValueConverter* valueConverter)
	{
		if (m_ValueConverter != NULL)
		{
			m_ValueConverter->AutoRelease();
			m_ValueConverter = NULL;
		}
		m_ValueConverter = valueConverter;
		if (m_ValueConverter != NULL)
		{
			m_ValueConverter->AddRef();
		}
	}
	void Bind(CComVariant& propertyValue)
	{
		if (propertyValue.vt == VT_EMPTY) return;

		ATLASSERT(m_Target != NULL && _tcslen(m_TargetProperty) > 0);

		ComponentPropertyAccessor* property_accessor = CScriptletContainerFactory::get_instance()->CreatePropertyAccessor(m_Target->GetClassName());
		if (property_accessor != NULL)
		{
			CComVariant value = propertyValue;
			if (m_ValueConverter != NULL)
			{
				value = m_ValueConverter->Convert(&value);
			}
			property_accessor->SetProperty(m_Target, m_TargetProperty, value);
			property_accessor->AutoRelease();
		}
	}
	void Sync()
	{		
		CComVariant propertyValue = m_DataContext->GetProperty(m_SourceProperty);
		Bind(propertyValue);
	}
	//override
	void OnPropertyChanged(LPCTSTR propertyKey, IDataContext* dataContext)
	{
		CComVariant propertyValue = dataContext->GetProperty(m_SourceProperty);
		OnPropertyChanged(propertyKey, propertyValue);
	}
	void OnPropertyChanged(LPCTSTR propertyKey, CComVariant& propertyValue)
	{
		if (propertyKey != NULL)
		{
			if (_tcsicmp(m_SourceProperty, propertyKey) != 0)
			{
				return;
			}
		}		
		Bind(propertyValue);
	}
protected:
	CComponent* m_Target;
	CString m_TargetProperty;
	IDataContext* m_DataContext;
	PropertyBinder* m_Source;
	CString m_SourceProperty;
	IValueConverter* m_ValueConverter;

	void Close()
	{
		if (m_ValueConverter != NULL)
		{
			m_ValueConverter->AutoRelease();
			m_ValueConverter = NULL;
		}
		m_Source->RemovePropertyBindingListener(this);
	}
};